/*
    GNOME Commander - A GNOME based file manager 
    Copyright (C) 2001-2002 Marcus Bjurman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/ 
#include <config.h>
#include <pthread.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <unistd.h>
#include "gnome-cmd-includes.h"

#include "../pixmaps/menu_ftp_connect.xpm"
#include "../pixmaps/menu_ftp_disconnect.xpm"

#define LEFT 0
#define RIGHT 1

struct _GnomeCmdMainWinPrivate
{
	gboolean current_fs; // either LEFT or RIGHT
	
	GtkWidget *main_win;
	GtkWidget *vbox;
	GtkWidget *menubar;
	GtkWidget *hpaned;
	GtkWidget *buttonbox;
	GtkWidget *file_selector[2];
	GtkAccelGroup *accel_group;

	GtkWidget *help_btn;
	GtkWidget *refresh_btn;
	GtkWidget *view_btn;
	GtkWidget *edit_btn;
	GtkWidget *copy_btn;
	GtkWidget *move_btn;
	GtkWidget *mkdir_btn;
	GtkWidget *delete_btn;
	GtkWidget *find_btn;
	GtkWidget *quit_btn;

	GtkWidget *toolbar;
};

static GnomeAppClass *parent_class = NULL;



static GtkWidget *
add_buttonbox_button (char *label,
					  GnomeCmdMainWin *mw,
					  char *data_label,
					  GtkAccelGroup *accel_group,
					  gint accel_signal_id)
{
	GtkWidget *button;

	button = gtk_button_new_with_label (label);
	gtk_widget_ref (button);
	gtk_object_set_data_full (GTK_OBJECT (main_win),
							  data_label,
							  button,
							  (GtkDestroyNotify) gtk_widget_unref);

	gtk_widget_show (button);
	gtk_box_pack_start (GTK_BOX (mw->priv->buttonbox), button, TRUE, TRUE, 0);
	gtk_widget_add_accelerator (button, "clicked",
								accel_group, accel_signal_id,
								0, GTK_ACCEL_VISIBLE);
	return button;
}


static void
slide_set_100_0 (GtkMenu *menu, gpointer user_data)
{
	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
							GTK_WIDGET (main_win)->allocation.width);	
}


static void
slide_set_80_20 (GtkMenu *menu, gpointer user_data)
{
	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
							(int)(GTK_WIDGET (main_win)->allocation.width*0.8f));	
}


static void
slide_set_60_40 (GtkMenu *menu, gpointer user_data)
{
	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
							(int)(GTK_WIDGET (main_win)->allocation.width*0.6f));	
}


static void
slide_set_50_50 (GtkMenu *menu, gpointer user_data)
{
	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
							(int)(GTK_WIDGET (main_win)->allocation.width*0.5f));	
}


static void
slide_set_40_60 (GtkMenu *menu, gpointer user_data)
{
	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
							(int)(GTK_WIDGET (main_win)->allocation.width*0.4f));	
}


static void
slide_set_20_80 (GtkMenu *menu, gpointer user_data)
{
	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
							(int)(GTK_WIDGET (main_win)->allocation.width*0.2f));	
}


static void
slide_set_0_100 (GtkMenu *menu, gpointer user_data)
{
	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned), 0);	
}


static GtkWidget *
create_slide_popup ()
{
	gint i;
	GtkWidget *menu;
	
	static GnomeUIInfo popmenu_uiinfo[] =
	{
		{
			GNOME_APP_UI_ITEM, N_("100 - 0"),
			NULL,
			slide_set_100_0, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("80 - 20"),
			NULL,
			slide_set_80_20, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("60 - 40"),
			NULL,
			slide_set_60_40, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("50 - 50"),
			NULL,
			slide_set_50_50, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("40 - 60"),
			NULL,
			slide_set_40_60, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("20 - 80"),
			NULL,
			slide_set_20_80, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("0 - 100"),
			NULL,
			slide_set_0_100, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		}
	};

	/* Set default callback data 
	 */
	i = 0;
	while (popmenu_uiinfo[i].type != GNOME_APP_UI_ENDOFINFO) {
		if (popmenu_uiinfo[i].type == GNOME_APP_UI_ITEM) {
			popmenu_uiinfo[i].user_data = main_win;
		}
		i++;
	}

	menu = gtk_menu_new ();
	gtk_widget_ref (menu);
	gtk_object_set_data_full (GTK_OBJECT (main_win),
							  "slide-popup", menu,
							  (GtkDestroyNotify)gtk_widget_unref);
	
	/* Fill the menu
	 */
	gnome_app_fill_menu (GTK_MENU_SHELL (menu), popmenu_uiinfo,
						 NULL, FALSE, 0);

	return GTK_WIDGET (menu);	
}


/*****************************************************************************
	Buttonbar callbacks
*****************************************************************************/

static void
on_help_clicked                        (GtkButton       *button,
                                        GnomeCmdMainWin *mw)
{
	help_help (NULL, mw);
}


static void
on_refresh_clicked                        (GtkButton       *button,
									   GnomeCmdMainWin *mw)
{
	view_refresh (NULL, mw);
}


static void
on_view_clicked                        (GtkButton       *button,
                                        GnomeCmdMainWin *mw)
{
	file_view (NULL, mw);
}


static void
on_edit_clicked                        (GtkButton       *button,
                                        GnomeCmdMainWin *mw)
{
	file_edit (NULL, mw);
}


static void
on_copy_clicked                        (GtkButton       *button,
                                        GnomeCmdMainWin *mw)
{
	file_copy (NULL, mw);
}


static void
on_move_clicked                        (GtkButton       *button,
                                        GnomeCmdMainWin *mw)
{
	file_move (NULL, mw);
}


static void
on_mkdir_clicked                       (GtkButton       *button,
                                        GnomeCmdMainWin *mw)
{
	file_mkdir (NULL, mw);
}


static void
on_delete_clicked                      (GtkButton       *button,
                                        GnomeCmdMainWin *mw)
{
	file_delete (NULL, mw);
}


static void
on_search_clicked                      (GtkButton       *button,
                                        GnomeCmdMainWin *mw)
{
	edit_search (NULL, mw);
}


static void
on_quit_clicked                        (GtkButton       *button,
                                        GnomeCmdMainWin *mw)
{
    gtk_main_quit ();
}


/*****************************************************************************
	Misc widgets callbacks
*****************************************************************************/

static void
on_slide_button_press (GtkWidget *widget, GdkEventButton *event, GnomeCmdMainWin *mw)
{
	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
		GtkPaned *paned = GTK_PANED (mw->priv->hpaned);

		/* Check that the handle was clicked and not one of the children */
		if (paned->handle == event->window)
			gtk_menu_popup (GTK_MENU(create_slide_popup ()), NULL, NULL, NULL, NULL, 
							event->button, event->time);
	}
}

static void
on_main_win_realize                    (GtkWidget       *widget,
                                        GnomeCmdMainWin *mw)
{
	IMAGE_init (widget);

	gnome_cmd_file_selector_set_active (
		gnome_cmd_main_win_get_left_fs (mw), TRUE);
	
	gnome_cmd_cmdline_set_dir (
		cmdline,
		gnome_cmd_dir_get_path (
			gnome_cmd_file_selector_get_directory (
				gnome_cmd_main_win_get_left_fs (mw))));
	
	gdk_window_set_icon (GTK_WIDGET (main_win)->window, NULL,
						 IMAGE_get_pixmap (PIXMAP_LOGO),
						 IMAGE_get_mask (PIXMAP_LOGO));
}


static void
on_main_win_size_allocate              (GtkWidget       *widget,
                                        GtkAllocation   *allocation,
                                        GnomeCmdMainWin *mw)
{
	gtk_paned_set_position (GTK_PANED (mw->priv->hpaned), allocation->width/2 - 5);
}


static void
on_left_fs_select                      (GtkCList *list,
										GdkEventButton *event,
                                        GnomeCmdMainWin *mw)
{
		mw->priv->current_fs = LEFT;
		
		gnome_cmd_file_selector_set_active (
			GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT]), TRUE);
		gnome_cmd_file_selector_set_active (
			GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT]), FALSE);
}



static void
on_right_fs_select                     (GtkCList *list,
										GdkEventButton *event,
                                        GnomeCmdMainWin *mw)
{
		mw->priv->current_fs = RIGHT;
		
		gnome_cmd_file_selector_set_active (
			GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT]), TRUE);
		gnome_cmd_file_selector_set_active (
			GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT]), FALSE);
}


static void
on_fs_list_resize_column            (GtkCList        *clist,
                                     gint             column,
                                     gint             width,
                                     GtkCList        *other_clist)
{
	static gboolean column_resize_lock = FALSE;

    /* the lock is used so that we dont get into the situation where
       the left list triggers the right witch triggers the left ... */
    if (!column_resize_lock)
    {
		column_resize_lock = TRUE;
		gnome_cmd_data_set_fs_col_width ((guint)column, width);
		gtk_clist_set_column_width (other_clist, column, width);
		column_resize_lock = FALSE;
    }
}


static void
on_size_allocate                  (GtkWidget *widget,
								   GtkAllocation *allocation,
								   gpointer user_data)
{
	gnome_cmd_data_set_position (allocation->width, allocation->height);
}



/*******************************
 * Gtk class implementation
 *******************************/

static void
destroy (GtkObject *object)
{
	gtk_main_quit ();
}


static void
map (GtkWidget *widget)
{
	if (GTK_WIDGET_CLASS (parent_class)->map != NULL)
		GTK_WIDGET_CLASS (parent_class)->map (widget);
}


static void
class_init (GnomeCmdMainWinClass *class)
{
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;

	object_class = GTK_OBJECT_CLASS (class);
	widget_class = GTK_WIDGET_CLASS (class);

	parent_class = gtk_type_class (gnome_app_get_type ());

	object_class->destroy = destroy;

	widget_class->map = map;
}


static void
init (GnomeCmdMainWin *mw)
{
	gint main_win_width, main_win_height;
	
	static GnomeUIInfo files_menu_uiinfo[] =
	{
		{
			GNOME_APP_UI_ITEM, N_("Change owner/group"),
			NULL,
			file_chown, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Change permissions"),
			NULL,
			file_chmod, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Advanced rename tool"),
			NULL,
			file_advrename, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			GDK_T, GDK_CONTROL_MASK, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Properties..."),
			NULL,
			file_properties, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			GDK_Return, GDK_MOD1_MASK, NULL
		},
		GNOMEUIINFO_SEPARATOR,
		{
			GNOME_APP_UI_ITEM, N_("Diff"),
			NULL,
			file_diff, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		GNOMEUIINFO_SEPARATOR,
		{
			GNOME_APP_UI_ITEM, N_("Quit"),
			NULL,
			file_exit, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_QUIT,
			GDK_Q, GDK_CONTROL_MASK, NULL
		},
		GNOMEUIINFO_END
	};

	static GnomeUIInfo mark_menu_uiinfo[] =
	{
		{
			GNOME_APP_UI_ITEM, N_("Select all"),
			NULL,
			mark_select_all, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			GDK_KP_Add, GDK_CONTROL_MASK, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Unselect all"),
			NULL,
			mark_unselect_all, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			GDK_KP_Subtract, GDK_CONTROL_MASK, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Select with pattern"),
			NULL,
			mark_select_with_pattern, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			GDK_KP_Add, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Unselect with pattern"),
			NULL,
			mark_unselect_with_pattern, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			GDK_KP_Subtract, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Invert selection"),
			NULL,
			mark_invert_selection, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			GDK_KP_Multiply, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Restore selection"),
			NULL,
			mark_restore_selection, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			GDK_KP_Divide, 0, NULL
		},
		GNOMEUIINFO_END
	};
	
	//static GnomeUIInfo commands_menu_uiinfo[] =
	//{
	//};

	static GnomeUIInfo edit_menu_uiinfo[] =
	{
		GNOMEUIINFO_MENU_CUT_ITEM (file_cap_cut, NULL),
		GNOMEUIINFO_MENU_COPY_ITEM (file_cap_copy, NULL),
		GNOMEUIINFO_MENU_PASTE_ITEM (file_cap_paste, NULL),
		{
			GNOME_APP_UI_ITEM, N_("Delete"),
			NULL,
			file_delete, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		GNOMEUIINFO_SEPARATOR,
		{
			GNOME_APP_UI_ITEM, N_("Search..."),
			NULL,
			edit_search, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
			GDK_F7, GDK_MOD1_MASK, NULL
		},
		GNOMEUIINFO_END
	};

	static GnomeUIInfo view_menu_uiinfo[] =
	{
		{
			GNOME_APP_UI_ITEM, N_("Refresh"),
			NULL,
			view_refresh, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REFRESH,
			GDK_R, GDK_CONTROL_MASK, NULL
		},
		GNOMEUIINFO_END
	};
	
	static GnomeUIInfo options_menu_uiinfo[] =
	{
		{
			GNOME_APP_UI_ITEM, N_("Options..."),
			NULL,
			options_edit, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
			GDK_O, GDK_CONTROL_MASK, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Save options"),
			NULL,
			options_save, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Save position"),
			NULL,
			options_save_position, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE,
			0, 0, NULL
		},
		GNOMEUIINFO_END
	};
	static GnomeUIInfo ftp_menu_uiinfo[] =
	{
		{
			GNOME_APP_UI_ITEM, N_("Connect..."),
			NULL,
			ftp_connect, NULL, NULL,
			GNOME_APP_PIXMAP_DATA, menu_ftp_connect_xpm,
			GDK_F, GDK_CONTROL_MASK, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_("Disconnect"),
			NULL,
			ftp_disconnect, NULL, NULL,
			GNOME_APP_PIXMAP_DATA, menu_ftp_disconnect_xpm,
			GDK_F, GDK_CONTROL_MASK|GDK_SHIFT_MASK, NULL
		},
		GNOMEUIINFO_END
	};

	static GnomeUIInfo help_menu_uiinfo[] =
	{
		{
			GNOME_APP_UI_ITEM, N_("User manual..."),
			NULL,
			help_help, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_RED,
			GDK_H, GDK_CONTROL_MASK, NULL
		},
		GNOMEUIINFO_SEPARATOR,
		GNOMEUIINFO_MENU_ABOUT_ITEM (help_about, NULL),
		GNOMEUIINFO_END
	};

	static GnomeUIInfo menubar_uiinfo[] =
	{
		{
			GNOME_APP_UI_SUBTREE, N_("_Files"),
			NULL,
			files_menu_uiinfo, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		GNOMEUIINFO_MENU_EDIT_TREE (edit_menu_uiinfo),
		{
			GNOME_APP_UI_SUBTREE, N_("_Mark"),
			NULL,
			mark_menu_uiinfo, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		GNOMEUIINFO_MENU_VIEW_TREE (view_menu_uiinfo),
		GNOMEUIINFO_MENU_SETTINGS_TREE (options_menu_uiinfo),
		{
			GNOME_APP_UI_SUBTREE, N_("F_TP"),
			NULL,
			ftp_menu_uiinfo, NULL, NULL,
			GNOME_APP_PIXMAP_NONE, NULL,
			0, 0, NULL
		},
		GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
		GNOMEUIINFO_END
	};

	static GnomeUIInfo toolbar_uiinfo[] =
	{
		{
			GNOME_APP_UI_ITEM, N_(NULL),
			N_("File properties..."),
			file_properties, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PROPERTIES,
			0, 0, NULL
 		},
		GNOMEUIINFO_SEPARATOR,
		{
			GNOME_APP_UI_ITEM, N_(NULL),
			N_("Cut"),
			file_cap_cut, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_CUT,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_(NULL),
			N_("Copy"),
			file_cap_copy, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_COPY,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_(NULL),
			N_("Paste"),
			file_cap_paste, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PASTE,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_(NULL),
			N_("Delete"),
			file_delete, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_TRASH,
			0, 0, NULL
		},
		GNOMEUIINFO_SEPARATOR,
		{
			GNOME_APP_UI_ITEM, N_(NULL),
			N_("Refresh"),
			view_refresh, NULL, NULL,
			GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_REFRESH,
			0, 0, NULL
		},
		GNOMEUIINFO_SEPARATOR,
		{
			GNOME_APP_UI_ITEM, N_(NULL),
			N_("ftp connect"),
			ftp_connect, NULL, NULL,
			GNOME_APP_PIXMAP_DATA, menu_ftp_connect_xpm,
			0, 0, NULL
		},
		{
			GNOME_APP_UI_ITEM, N_(NULL),
			N_("ftp disconnect"),
			ftp_disconnect, NULL, NULL,
			GNOME_APP_PIXMAP_DATA, menu_ftp_disconnect_xpm,
			0, 0, NULL
		},
		GNOMEUIINFO_END
	};

	int i;

	/* It is very important that this global variable gets assigned here so that
	 * child widgets to this window can use that variable when initializing
	 */
	main_win = GNOME_CMD_MAIN_WIN (mw);

	i = 0;
	while (files_menu_uiinfo[i].type != GNOME_APP_UI_ENDOFINFO) {
		if (files_menu_uiinfo[i].type == GNOME_APP_UI_ITEM
			|| files_menu_uiinfo[i].type == GNOME_APP_UI_ITEM_CONFIGURABLE) {
			files_menu_uiinfo[i].user_data = mw;
		}
		i++;
	}

	i = 0;
	while (mark_menu_uiinfo[i].type != GNOME_APP_UI_ENDOFINFO) {
		if (mark_menu_uiinfo[i].type == GNOME_APP_UI_ITEM
			|| mark_menu_uiinfo[i].type == GNOME_APP_UI_ITEM_CONFIGURABLE) {
			mark_menu_uiinfo[i].user_data = mw;
		}
		i++;
	}

	i = 0;
	while (edit_menu_uiinfo[i].type != GNOME_APP_UI_ENDOFINFO) {
		if (edit_menu_uiinfo[i].type == GNOME_APP_UI_ITEM
			|| edit_menu_uiinfo[i].type == GNOME_APP_UI_ITEM_CONFIGURABLE) {
			edit_menu_uiinfo[i].user_data = mw;
		}
		i++;
	}

	i = 0;
	while (view_menu_uiinfo[i].type != GNOME_APP_UI_ENDOFINFO) {
		if (view_menu_uiinfo[i].type == GNOME_APP_UI_ITEM
			|| view_menu_uiinfo[i].type == GNOME_APP_UI_ITEM_CONFIGURABLE) {
			view_menu_uiinfo[i].user_data = mw;
		}
		i++;
	}

	i = 0;
	while (options_menu_uiinfo[i].type != GNOME_APP_UI_ENDOFINFO) {
		if (options_menu_uiinfo[i].type == GNOME_APP_UI_ITEM
			|| options_menu_uiinfo[i].type == GNOME_APP_UI_ITEM_CONFIGURABLE) {
			options_menu_uiinfo[i].user_data = mw;
		}
		i++;
	}

	i = 0;
	while (ftp_menu_uiinfo[i].type != GNOME_APP_UI_ENDOFINFO) {
		if (ftp_menu_uiinfo[i].type == GNOME_APP_UI_ITEM
			|| ftp_menu_uiinfo[i].type == GNOME_APP_UI_ITEM_CONFIGURABLE) {
			ftp_menu_uiinfo[i].user_data = mw;
		}
		i++;
	}

	i = 0;
	while (help_menu_uiinfo[i].type != GNOME_APP_UI_ENDOFINFO) {
		if (help_menu_uiinfo[i].type == GNOME_APP_UI_ITEM
			|| help_menu_uiinfo[i].type == GNOME_APP_UI_ITEM_CONFIGURABLE) {
			help_menu_uiinfo[i].user_data = mw;
		}
		i++;
	}

	i = 0;
	while (toolbar_uiinfo[i].type != GNOME_APP_UI_ENDOFINFO) {
		if (toolbar_uiinfo[i].type == GNOME_APP_UI_ITEM
			|| toolbar_uiinfo[i].type == GNOME_APP_UI_ITEM_CONFIGURABLE) {
			toolbar_uiinfo[i].user_data = mw;
		}
		i++;
	}
	
	mw->priv = g_new (GnomeCmdMainWinPrivate, 1);
	mw->priv->current_fs = LEFT;
	mw->priv->accel_group = gtk_accel_group_new ();

	gnome_cmd_data_get_position (&main_win_width, &main_win_height);

	gtk_object_set_data (GTK_OBJECT (main_win), "main_win", main_win);
	gtk_window_set_title (GTK_WINDOW (main_win), "GNOME Commander");
	gtk_window_set_position (GTK_WINDOW (main_win), GTK_WIN_POS_CENTER);
	gtk_window_set_policy (GTK_WINDOW (main_win), TRUE, TRUE, FALSE);
	gtk_widget_set_usize (GTK_WIDGET (main_win), main_win_width, main_win_height);

	mw->priv->vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (mw->priv->vbox);
	gtk_object_set_data_full (GTK_OBJECT (main_win), "vbox", mw->priv->vbox,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (mw->priv->vbox);
	gnome_app_set_contents (GNOME_APP (main_win), mw->priv->vbox);
	gnome_app_create_menus (GNOME_APP (mw), menubar_uiinfo);
	gnome_app_create_toolbar (GNOME_APP (mw), toolbar_uiinfo);

	mw->priv->hpaned = gtk_hpaned_new ();
	gtk_widget_ref (mw->priv->hpaned);
	gtk_object_set_data_full (GTK_OBJECT (main_win), "paned", mw->priv->hpaned,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (mw->priv->hpaned);
	gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->hpaned,
						TRUE, TRUE, 0);
	gtk_paned_set_handle_size (GTK_PANED (mw->priv->hpaned), 8);
	gtk_paned_set_gutter_size (GTK_PANED (mw->priv->hpaned), 10);

	cmdline = GNOME_CMD_CMDLINE (gnome_cmd_cmdline_new ());
	gtk_widget_ref (GTK_WIDGET (cmdline));
	gtk_object_set_data_full (GTK_OBJECT (main_win), "cmdline", GTK_WIDGET (cmdline),
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (GTK_WIDGET (cmdline));
	gtk_box_pack_start (GTK_BOX (mw->priv->vbox), GTK_WIDGET (cmdline), FALSE, TRUE, 0);

	mw->priv->buttonbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (mw->priv->buttonbox);
	gtk_object_set_data_full (GTK_OBJECT (main_win), "buttonbox", mw->priv->buttonbox,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (mw->priv->buttonbox);
	gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->buttonbox, FALSE, TRUE, 0);

	mw->priv->help_btn = add_buttonbox_button(
		_("F1 Help"), main_win, "help_btn", mw->priv->accel_group, 0);
	mw->priv->refresh_btn = add_buttonbox_button(
		_("F2 Refresh"), main_win, "run_btn", mw->priv->accel_group, 0);
	mw->priv->view_btn = add_buttonbox_button(
		_("F3 View"), main_win, "view_btn", mw->priv->accel_group, 0);
	mw->priv->edit_btn = add_buttonbox_button(
		_("F4 Edit"), main_win, "edit_btn", mw->priv->accel_group, 0);
	mw->priv->copy_btn = add_buttonbox_button(
		_("F5 Copy"), main_win, "copy_btn", mw->priv->accel_group, 0);
	mw->priv->move_btn = add_buttonbox_button(
		_("F6 Move"), main_win, "move_btn", mw->priv->accel_group, 0);
	mw->priv->mkdir_btn = add_buttonbox_button(
		_("F7 Mkdir"), main_win, "mkdir_btn", mw->priv->accel_group, 0);
	mw->priv->delete_btn = add_buttonbox_button(
		_("F8 Delete"), main_win, "delete_btn", mw->priv->accel_group, 0);
	mw->priv->find_btn = add_buttonbox_button(
		_("F9 Search"), main_win, "find_btn", mw->priv->accel_group, 0);
	mw->priv->quit_btn = add_buttonbox_button(
		_("F10 Quit"), main_win, "quit_btn", mw->priv->accel_group, 0);

	GTK_WIDGET_UNSET_FLAGS (mw->priv->help_btn, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (mw->priv->refresh_btn, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (mw->priv->view_btn, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (mw->priv->edit_btn, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (mw->priv->copy_btn, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (mw->priv->move_btn, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (mw->priv->mkdir_btn, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (mw->priv->delete_btn, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (mw->priv->find_btn, GTK_CAN_FOCUS);
	GTK_WIDGET_UNSET_FLAGS (mw->priv->quit_btn, GTK_CAN_FOCUS);
	
	mw->priv->file_selector[LEFT] = gnome_cmd_file_selector_new ();
	gtk_widget_ref (mw->priv->file_selector[LEFT]);
	gtk_object_set_data_full (GTK_OBJECT (main_win), "left_file_selector",
							  mw->priv->file_selector[LEFT],
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (mw->priv->file_selector[LEFT]);
	gtk_paned_pack1 (GTK_PANED (mw->priv->hpaned), mw->priv->file_selector[LEFT],
					 TRUE, TRUE);

	mw->priv->file_selector[RIGHT] = gnome_cmd_file_selector_new ();
	gtk_widget_ref (mw->priv->file_selector[RIGHT]);
	gtk_object_set_data_full (GTK_OBJECT (main_win), "left_file_selector",
							  mw->priv->file_selector[RIGHT],
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (mw->priv->file_selector[RIGHT]);
	gtk_paned_pack2 (GTK_PANED (mw->priv->hpaned), mw->priv->file_selector[RIGHT],
					 TRUE, TRUE);

	gnome_cmd_connection_create_local ();

	gnome_cmd_file_selector_set_connection (gnome_cmd_main_win_get_left_fs (mw),
											gnome_cmd_connection_get_local ());
	gnome_cmd_file_selector_set_connection (gnome_cmd_main_win_get_right_fs (mw),
											gnome_cmd_connection_get_local ());

	gtk_widget_modify_style (GTK_BIN (mw->priv->help_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->refresh_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->view_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->edit_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->copy_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->move_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->mkdir_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->delete_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->find_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->quit_btn)->child, main_rc_style);

	gtk_button_set_relief (GTK_BUTTON (mw->priv->help_btn), GTK_RELIEF_HALF);
	gtk_button_set_relief (GTK_BUTTON (mw->priv->refresh_btn), GTK_RELIEF_HALF);
	gtk_button_set_relief (GTK_BUTTON (mw->priv->view_btn), GTK_RELIEF_HALF);
	gtk_button_set_relief (GTK_BUTTON (mw->priv->edit_btn), GTK_RELIEF_HALF);
	gtk_button_set_relief (GTK_BUTTON (mw->priv->copy_btn), GTK_RELIEF_HALF);
	gtk_button_set_relief (GTK_BUTTON (mw->priv->move_btn), GTK_RELIEF_HALF);
	gtk_button_set_relief (GTK_BUTTON (mw->priv->mkdir_btn), GTK_RELIEF_HALF);
	gtk_button_set_relief (GTK_BUTTON (mw->priv->delete_btn), GTK_RELIEF_HALF);
	gtk_button_set_relief (GTK_BUTTON (mw->priv->find_btn), GTK_RELIEF_HALF);
	gtk_button_set_relief (GTK_BUTTON (mw->priv->quit_btn), GTK_RELIEF_HALF);

	
	gtk_signal_connect (GTK_OBJECT (main_win), "realize",
						GTK_SIGNAL_FUNC (on_main_win_realize), mw);
	gtk_signal_connect (GTK_OBJECT (main_win), "size_allocate",
						GTK_SIGNAL_FUNC (on_main_win_size_allocate), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->help_btn), "clicked",
						GTK_SIGNAL_FUNC (on_help_clicked), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->refresh_btn), "clicked",
						GTK_SIGNAL_FUNC (on_refresh_clicked), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->view_btn), "clicked",
						GTK_SIGNAL_FUNC (on_view_clicked), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->edit_btn), "clicked",
						GTK_SIGNAL_FUNC (on_edit_clicked), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->copy_btn), "clicked",
						GTK_SIGNAL_FUNC (on_copy_clicked), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->move_btn), "clicked",
						GTK_SIGNAL_FUNC (on_move_clicked), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->mkdir_btn), "clicked",
						GTK_SIGNAL_FUNC (on_mkdir_clicked), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->delete_btn), "clicked",
						GTK_SIGNAL_FUNC (on_delete_clicked), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->find_btn), "clicked",
						GTK_SIGNAL_FUNC (on_search_clicked), mw);
	gtk_signal_connect (GTK_OBJECT (mw->priv->quit_btn), "clicked",
						GTK_SIGNAL_FUNC (on_quit_clicked), mw);

	gtk_signal_connect (
		GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->list),
		"resize_column", GTK_SIGNAL_FUNC (on_fs_list_resize_column),
		GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->list);
	gtk_signal_connect (
		GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->list),
		"resize_column", GTK_SIGNAL_FUNC (on_fs_list_resize_column),
		GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->list);
	gtk_signal_connect (
		GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT])->list),
		"button_press_event", GTK_SIGNAL_FUNC (on_left_fs_select), mw);
	gtk_signal_connect (
		GTK_OBJECT (GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT])->list),
		"button_press_event", GTK_SIGNAL_FUNC (on_right_fs_select), mw);
	gtk_signal_connect (
		GTK_OBJECT (mw), "size-allocate",
		GTK_SIGNAL_FUNC (on_size_allocate), mw);
	gtk_signal_connect (
		GTK_OBJECT (mw->priv->hpaned),
		"button_press_event", GTK_SIGNAL_FUNC (on_slide_button_press), mw);	

	gtk_window_add_accel_group (GTK_WINDOW (main_win), mw->priv->accel_group);
}


GtkType
gnome_cmd_main_win_get_type         (void)
{
	static GtkType mw_type = 0;

	if (mw_type == 0)
	{
		GtkTypeInfo mw_info =
		{
			"GnomeCmdMainWin",
			sizeof (GnomeCmdMainWin),
			sizeof (GnomeCmdMainWinClass),
			(GtkClassInitFunc) class_init,
			(GtkObjectInitFunc) init,
			/* reserved_1 */ NULL,
			/* reserved_2 */ NULL,
			(GtkClassInitFunc) NULL
		};

		mw_type = gtk_type_unique (gnome_app_get_type (), &mw_info);
	}
	return mw_type;
}



GtkWidget*
gnome_cmd_main_win_new              ()
{
	GnomeCmdMainWin *mw;

	mw = gtk_type_new (gnome_cmd_main_win_get_type ());

	return GTK_WIDGET (mw);
}


GnomeCmdFileSelector*
gnome_cmd_main_win_get_active_fs         (GnomeCmdMainWin *mw)
{
	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[mw->priv->current_fs]);
}


GnomeCmdFileSelector*
gnome_cmd_main_win_get_inactive_fs         (GnomeCmdMainWin *mw)
{
	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[!mw->priv->current_fs]);
}


GnomeCmdFileSelector*
gnome_cmd_main_win_get_left_fs         (GnomeCmdMainWin *mw)
{
	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT]);
}


GnomeCmdFileSelector*
gnome_cmd_main_win_get_right_fs         (GnomeCmdMainWin *mw)
{
	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT]);
}


void
gnome_cmd_main_win_update_style          (GnomeCmdMainWin *mw)
{
	g_return_if_fail (mw != NULL);
	g_return_if_fail (mw->priv != NULL);
	
	gtk_widget_modify_style (GTK_BIN (mw->priv->help_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->refresh_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->view_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->edit_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->copy_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->move_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->mkdir_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->delete_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->find_btn)->child, main_rc_style);
	gtk_widget_modify_style (GTK_BIN (mw->priv->quit_btn)->child, main_rc_style);

	IMAGE_clear_mime_cache ();

	gnome_cmd_file_selector_update_style (gnome_cmd_main_win_get_left_fs (mw));
	gnome_cmd_file_selector_update_style (gnome_cmd_main_win_get_right_fs (mw));

	gnome_cmd_cmdline_update_style (cmdline);
}


void
gnome_cmd_main_win_new_cwd               (GnomeCmdMainWin *mw,
										  const gchar *cwd)
{
	gnome_cmd_cmdline_set_dir (cmdline, cwd);
}


void
gnome_cmd_main_win_focus_cmdline         (GnomeCmdMainWin *mw)
{
	gnome_cmd_cmdline_focus (cmdline);
}


void
gnome_cmd_main_win_focus_file_lists      (GnomeCmdMainWin *mw)
{
	gnome_cmd_file_selector_set_active (
		gnome_cmd_main_win_get_active_fs (mw), TRUE);
	gnome_cmd_file_selector_set_active (
		gnome_cmd_main_win_get_inactive_fs (mw), FALSE);
}


gboolean
gnome_cmd_main_win_keypressed            (GnomeCmdMainWin *mw,
										  GdkEventKey *event)
{
	if (state_is_alt (event->state))
	{
		switch (event->keyval)
		{
			case GDK_1:
			{
				GnomeCmdFileSelector *fs = gnome_cmd_main_win_get_left_fs (mw);
 				gnome_cmd_combo_popup_list (GNOME_CMD_COMBO (fs->con_combo), fs->list);
			}
			return TRUE;
			
			case GDK_2:
			{
				GnomeCmdFileSelector *fs = gnome_cmd_main_win_get_right_fs (mw);
 				gnome_cmd_combo_popup_list (GNOME_CMD_COMBO (fs->con_combo), fs->list);
			}
			return TRUE;

			case GDK_F8:
				gnome_cmd_cmdline_show_history (cmdline);				
				return TRUE;

			case GDK_KP_Add:
				mark_select_all_with_same_extension (NULL, mw);
				break;
		}
	}
	else if (state_is_ctrl (event->state))
	{
		switch (event->keyval)
		{
			case GDK_u:
			case GDK_U:
			{
				GnomeCmdDir *dir1 = gnome_cmd_file_selector_get_directory (
					gnome_cmd_main_win_get_left_fs (mw));
				GnomeCmdDir *dir2 = gnome_cmd_file_selector_get_directory (
					gnome_cmd_main_win_get_right_fs (mw));

				gnome_cmd_dir_ref (dir1);
				gnome_cmd_dir_ref (dir2);

				gnome_cmd_file_selector_set_directory (
					gnome_cmd_main_win_get_left_fs (mw), dir2);
				gnome_cmd_file_selector_set_directory (
					gnome_cmd_main_win_get_right_fs (mw), dir1);

				gnome_cmd_dir_unref (dir1);
				gnome_cmd_dir_unref (dir2);

				clear_event_key (event);
			}
			return TRUE;
		}
	}
	else if (state_is_blank (event->state))
	{	
		switch (event->keyval)
		{
			case GDK_Tab:
				/* hack to avoid the deafult handling of the tab-key */
				clear_event_key (event);
				mw->priv->current_fs = !mw->priv->current_fs;
				gnome_cmd_file_selector_set_active (
					gnome_cmd_main_win_get_active_fs (mw), TRUE);
				gnome_cmd_file_selector_set_active (
					gnome_cmd_main_win_get_inactive_fs (mw), FALSE);
				gnome_cmd_cmdline_set_dir (
					cmdline,
					gnome_cmd_dir_get_path (
						gnome_cmd_file_selector_get_directory (
							gnome_cmd_main_win_get_active_fs (main_win))));
				return TRUE;
				
			case GDK_F1:
				on_help_clicked (NULL, mw);
				return TRUE;
			case GDK_F2:
				on_refresh_clicked (NULL, mw);
				return TRUE;
			case GDK_F3:
				on_view_clicked (NULL, mw);
				return TRUE;
			case GDK_F4:
				on_edit_clicked (NULL, mw);
				return TRUE;
			case GDK_F5:
				on_copy_clicked (NULL, mw);
				return TRUE;
			case GDK_F6:
				on_move_clicked (NULL, mw);
				return TRUE;
			case GDK_F7:
				on_mkdir_clicked (NULL, mw);
				return TRUE;
			case GDK_F8:
				on_delete_clicked (NULL, mw);
				return TRUE;
			case GDK_F9:
				on_search_clicked (NULL, mw);
				return TRUE;
			case GDK_F10:
				on_quit_clicked (NULL, mw);
				return TRUE;
		}
	}

	if (gnome_cmd_file_selector_keypressed (
			gnome_cmd_main_win_get_active_fs (mw), event))
		return TRUE;
	
	return FALSE;
}



